/*	$LAAS$ */
/*
 * Copyright (c) 2005 CNRS/LAAS
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

typedef struct fh2805dev {
	int fd;
	int readTask;
	char* buf;
	int (*decodeFunc)(struct fh2805dev *);
	char asyncAnswer;
	int pos;
	int speed;
	int refPos;
	int refSpeed;
	unsigned int maxSpeed;
	unsigned int acceleration;
	int p;
	int i;
	unsigned int status;
	int temperature;
	char *version;
	pthread_mutex_t *mutex;
} fh2805dev_t;

#define FH2805_READ_TASK_NAME "fh2805ReadTask"
#define FH2805_READ_TASK_PRIORITY 20
#define FH2805_READ_TASK_STACK_SIZE 32768

#define FH2805_BUF_SIZE 256

/***
 *** Commands for Faulhaber 2805 motor controler
 ***/

typedef enum {
	ACTIVATE_POSITION_LIMITS,
	CAPTURE_HOMING_SEQUENCE,
	CLEAR_OUTPUT,
	DISABLE_DRIVE,
	DISABLE_PROGRAM,
	ENABLE_DRIVE,
	ENABLE_POSITIONNING_MODE,
	ENABLE_PROGRAM,
	ENABLE_VELOCITY_MODE,
	GET_ACCELERATION,
	GET_ACTUAL_HOMING_STATUS,
	GET_ACTUAL_POSITION,
	GET_ACTUAL_SPEED,
	GET_ACTUAL_STATUS,
	GET_CONTINUOUS_CURRENT,
	GET_CURRENT_LIMIT,
	GET_DELAYED_CURRENT_ERROR,
	GET_DEVIATION,
	GET_ENCODER_RESOLUTION,
	GET_ENHANCED_STATUS,
	GET_FAULT_STATUS,
	GET_HARD_STATUS,
	GET_HOMING_SEQUENCE_CONFIGURATION,
	GET_HOMING_SPEED,
	GET_INTEGRAL_TERM,
	GET_MAXIMUM_SPEED,
	GET_MINIMUM_ANALOG_VOLDATE,
	GET_MINIMUM_VELOCITY,
	GET_MODE,
	GET_MOTOR_RESISTANCE,
	GET_NEGATIVE_LIMIT,
	GET_NODE_ADDRESS,
	GET_PEAK_CURRENT,
	GET_POSITIVE_LIMIT,
	GET_PROGRAM_SEQUENCE,
	GET_PROPORTIONAL_TERM,
	GET_REAL_CURRENT,
	GET_SPECIAL_CONFIG_SET,
	GET_SPEED_CONSTANT,
	GET_STATUS,
	GET_STEP_NUMBER,
	GET_STEP_WIDTH,
	GET_TEMPERATURE,
	GET_VELOCITY,
	GET_VERSION,
	GO_HOMING_SEQUENCE,
	NOTIFY_ERROR,
	NOTIFY_POSITION,
	NOTIFY_POSITION_GOAL,
	NOTIFY_POSITION_OFF,
	NOTIFY_VELOCITY,
	NOTIFY_VELOCITY_OFF,
	POWER_ON_HOMING_SEQUENCE,
	PROGRAM_SEQUENCE_BEGIN,
	PROGRAM_SEQUENCE_END,
	RESET_FACTORY_CONFIG,
	RESET_NODE,
	SAVE_TO_EEPROM,
	SET_ABSOLUTE_POSITION_REFERENCE,
	SET_ACCELERATION,
	SET_ANALOG_DIRECTION_LEFT,
	SET_ANALOG_DIRECTION_RIGHT,
	SET_ANALOG_POS_MODE,
	SET_ASYNC_ANSWER,
	SET_BAUD,
	SET_CONTINUOUS_CURRENT_LIMIT,
	SET_CONTINUOUS_MODE,
	SET_DCR_MODE,
	SET_DELAYED_CURRENT_ERROR,
	SET_DEVIATION,
	SET_DIGITAL_OUTPUT,
	SET_DIRECTION_INPUT,
	SET_ENCODER_RESOLUTION,
	SET_ERROR_OUTPUT,
	SET_HARD_BLOCKING,
	SET_HARD_DIRECTION,
	SET_HARD_LIMIT,
	SET_HARD_NOTIFY,
	SET_HARD_POLARITY,
	SET_HOME_ARMING,
	SET_HOMING_SPEED,
	SET_INTEGRAL_TERM,
	SET_MAXIMUM_SPEED,
	SET_MINIMUM_ANALOG_VOLTAGE,
	SET_MINIMUM_VELOCITY,
	SET_MOTOR_RESISTANCE,
	SET_NODE_ADDRESS,
	SET_OUTPUT,
	SET_PEAK_CURRENT_LIMIT,
	SET_POSITION,
	SET_POSITION_RANGE_LIMITS,
	SET_POSITION_ZERO,
	SET_PROPORTIONAL_TERM,
	SET_REF_INPUT,
	SET_RELATIVE_POSITION_REFERENCE,
	SET_SPEED_CONSTANT,
	SET_STEPPERMOTOR_MODE,
	SET_STEP_NUMBER,
	SET_STEP_WIDTH,
	SET_VELOCITY_SOURCE,
	SET_VOLTAGE_MODE
} FH2805_CMD;

/* Home position on jido */
#define FH2805_JIDO_HOME_POSITION (-83000)

extern fh2805dev_t *fh2805Init(char *, int);
extern int fh2805End(fh2805dev_t *);
extern int fh2805Cmd(fh2805dev_t *, FH2805_CMD, ...);
extern int fh2805Busy(fh2805dev_t *);


